@@ -4,7 +4,7 @@ package WebService::Avalara::AvaTax;
 
 use Modern::Perl '2011';    ## no critic (Modules::ProhibitUseQuotedVersion)
 
-our $VERSION = '0.007';     # TRIAL VERSION
+our $VERSION = '0.008';     # TRIAL VERSION
 use utf8;
 
 #pod =head1 SYNOPSIS
@@ -21,210 +21,78 @@ use utf8;
 #pod This class provides a Perl method API for
 #pod L<Avalara AvaTax|http://developer.avalara.com/api-docs/soap>
 #pod web services. The first call to any AvaTax SOAP operation uses
-#pod L<XML::Compile::WSDL11|XML::Compile::WSDL11> to compile and execute against the
-#pod specified Avalara AvaTax service; subsequent calls can vary the
-#pod parameters but will use the same compiled code.
+#pod L<XML::Compile::WSDL11|XML::Compile::WSDL11>
+#pod to compile and execute against the specified Avalara AvaTax service;
+#pod subsequent calls can vary the parameters but will use the same compiled code.
 #pod
 #pod =cut
 
 use Carp;
 use Const::Fast;
 use English '-no_match_vars';
-use Log::Report;
-use LWPx::UserAgent::Cached;
 use Moo;
-use MooX::Types::MooseLike::Email 'EmailAddressLoose';
 use Package::Stash;
 use Scalar::Util 'blessed';
 use Sys::Hostname;
 use Types::Standard qw(ArrayRef Bool HashRef InstanceOf Str);
-use Types::URI 'Uri';
-use URI;
-use XML::Compile::SOAP::WSS;
-use XML::Compile::SOAP11;
-use XML::Compile::SOAP12;
-use XML::Compile::WSDL11;
-use XML::Compile::Transport::SOAPHTTP;
-use MooX::Struct EndpointLocator => [
-    wsdl_uri => [ is => 'ro', isa => Uri, required => 1, coerce => 1 ],
-    port     => [ is => 'ro', isa => Str, required => 1 ],
-    service  => [ is => 'ro', isa => Str, required => 1 ],
-];
+use WebService::Avalara::AvaTax::Service::Address;
+use WebService::Avalara::AvaTax::Service::Tax;
 use namespace::clean;
+with 'WebService::Avalara::AvaTax::Role::Connection';
 
 #pod =method new
 #pod
-#pod Builds a new AvaTax web service client. See the L</ATTRIBUTES> section for
-#pod description of its named parameters.
-#pod
-#pod =attr username
-#pod
-#pod The Avalara email address used for authentication. Required.
+#pod Builds a new AvaTax web service client. Since this class consumes the
+#pod L<WebService::Avalara::AvaTax::Role::Connection|WebService::Avalara::AvaTax::Role::Connection>
+#pod role, please consult that module's documentation for a full list of attributes
+#pod that can be set at construction.
 #pod
 #pod =cut
 
-has username => ( is => 'ro', isa => EmailAddressLoose, required => 1 );
-
-#pod =attr password
+#pod =attr services
 #pod
-#pod The password used for Avalara authentication. Required.
+#pod This module is really just a convenience wrapper around instances of
+#pod L<WebService::Avalara::AvaTax::Service::Address|WebService::Avalara::AvaTax::Service::Address>
+#pod and
+#pod L<WebService::Avalara::AvaTax::Service::Tax|WebService::Avalara::AvaTax::Service::Tax>
+#pod modules. As such this attribute is used to keep an array reference to
+#pod instances of both classes, with the following attributes from L</new>
+#pod passed to both:
+#pod
+#pod =over
+#pod
+#pod =item L<username|WebService::Avalara::AvaTax::Role::Connection/username>
+#pod
+#pod =item L<password|WebService::Avalara::AvaTax::Role::Connection/password>
+#pod
+#pod =item L<is_production|WebService::Avalara::AvaTax::Role::Connection/is_production>
+#pod
+#pod =item L<user_agent|WebService::Avalara::AvaTax::Role::Connection/user_agent>
+#pod
+#pod =item L<debug|WebService::Avalara::AvaTax::Role::Connection/debug>
+#pod
+#pod =back
 #pod
 #pod =cut
 
-has password => ( is => 'ro', isa => Str, required => 1 );
-
-#pod =attr is_production
-#pod
-#pod A boolean value that indicates whether to connect to the production AvaTax
-#pod services (true) or development (false). Defaults to false.
-#pod
-#pod =cut
-
-has is_production => ( is => 'ro', isa => Bool, default => 0 );
-
-{
-    ## no critic (Subroutines::ProhibitCallsToUndeclaredSubs)
-    ## no critic (Modules::RequireExplicitInclusion)
-    has _endpoints =>
-        ( is => 'lazy', isa => ArrayRef [ InstanceOf [EndpointLocator] ] );
-    const my %SOAP_PARAMS => (
-        address => { port => 'AddressSvcSoap', service => 'AddressSvc' },
-        tax     => { port => 'TaxSvcSoap',     service => 'TaxSvc' },
-    );
-
-    sub _build__endpoints {
-        my $self = shift;
-        my $uri_base
-            = 'https://'
-            . ( $self->is_production ? 'avatax' : 'development' )
-            . '.avalara.net';
-        return [
-            map { EndpointLocator->new($_) } (
-                {   wsdl_uri => "$uri_base/address/addresssvc.wsdl",
-                    %{ $SOAP_PARAMS{address} },
-                },
-                {   wsdl_uri => "$uri_base/tax/taxsvc.wsdl",
-                    %{ $SOAP_PARAMS{tax} },
-                },
-            ),
-        ];
-    }
-}
-
-#pod =attr debug
-#pod
-#pod When set to true, the L<Log::Report|Log::Report> dispatcher used by
-#pod L<XML::Compile|XML::Compile> and friends is set to I<DEBUG> mode.
-#pod
-#pod =cut
-
-has debug => (
-    is      => 'ro',
-    isa     => Bool,
-    default => 0,
-    trigger =>
-        sub { dispatcher( mode => ( $_[1] ? 'DEBUG' : 'NORMAL' ), 'ALL' ) },
-);
-
-#pod =attr user_agent
-#pod
-#pod An instance of an L<LWP::UserAgent|LWP::UserAgent> (sub-)class. You can
-#pod use your own subclass to add features such as caching or enhanced logging.
-#pod
-#pod If you do not specify a C<user_agent> then we default to an
-#pod L<LWPx::UserAgent::Cached|LWPx::UserAgent::Cached> with its C<ssl_opts>
-#pod parameter set to C<< {verify_hostname => 0} >>.
-#pod
-#pod =cut
-
-has user_agent => (
-    is      => 'lazy',
-    isa     => InstanceOf ['LWP::UserAgent'],
-    default => sub {
-        LWPx::UserAgent::Cached->new( ssl_opts => { verify_hostname => 0 } );
-    },
-);
-
-#pod =attr wsdl
-#pod
-#pod After construction, you can retrieve the created
-#pod L<XML::Compile::WSDL11|XML::Compile::WSDL11> instance.
-#pod
-#pod Example:
-#pod
-#pod     my $wsdl = $avatax->wsdl;
-#pod     my @soap_operations = map { $_->name } $wsdl->operations;
-#pod
-#pod =cut
-
-has wsdl => (
+has services => (
     is       => 'lazy',
-    isa      => InstanceOf ['XML::Compile::WSDL11'],
+    isa      => ArrayRef,
     init_arg => undef,
-);
-
-sub _build_wsdl {
-    my $self = shift;
-    my $wsdl = XML::Compile::WSDL11->new;
-    for ( map { $_->wsdl_uri } @{ $self->_endpoints } ) {
-        $wsdl->addWSDL( $self->user_agent->get($_)->content );
-    }
-    return $wsdl;
-}
-
-has _wss => (
-    is      => 'ro',
-    isa     => InstanceOf ['XML::Compile::SOAP::WSS'],
-    default => sub { XML::Compile::SOAP::WSS->new },
-);
-
-has _auth => ( is => 'lazy', isa => InstanceOf ['XML::Compile::WSS'] );
-
-sub _build__auth {
-    my $self = shift;
-    my $wss  = $self->_wss;
-    return $wss->basicAuth( map { ( $_ => $self->$_ ) }
-            qw(username password) );
-}
-
-has _transports => (
-    is      => 'lazy',
-    isa     => ArrayRef [ InstanceOf ['XML::Compile::Transport::SOAPHTTP'] ],
-    default => sub {
-        [ map { $_[0]->_make_transport($_) } @{ $_[0]->_endpoints } ];
+    default  => sub {
+        [ map { $_[0]->_new_service($_) } qw(Address Tax) ];
     },
 );
 
-sub _make_transport {
-    my ( $self, $endpoint_locator ) = @_;
-
-    my $wss  = $self->_wss;
-    my $wsdl = $self->wsdl;
-    my $auth = $self->_auth;
-
-    my %soap_params  = map { $_ => $endpoint_locator->$_ } qw(port service);
-    my $endpoint_uri = URI->new( $wsdl->endPoint(%soap_params) );
-    my $user_agent   = $self->user_agent;
-
-    $user_agent->add_handler(
-        request_prepare => sub {
-            $_[0]->header( SOAPAction =>
-                    $wsdl->operation( $self->_operation_name, %soap_params )
-                    ->soapAction );
-        },
-        (   m_method => 'POST',
-            map { ( "m_$_" => $endpoint_uri->$_ ) } qw(scheme host_port path),
-        ),
-    );
-
-    return XML::Compile::Transport::SOAPHTTP->new(
-        address    => $endpoint_uri,
-        user_agent => $self->user_agent,
+sub _new_service {
+    my $self  = shift;
+    my $class = __PACKAGE__ . '::Service::' . shift;
+    return $class->new(
+        map { ( $_ => $self->$_ ) }
+            qw(username password is_production user_agent debug),
     );
 }
 
-has _operation_name => ( is => 'rw', isa => Str, default => q{} );
-
 #pod =head1 SEE ALSO
 #pod
 #pod =over
@@ -248,11 +116,16 @@ has _operation_name => ( is => 'rw', isa => Str, default => q{} );
 #pod
 #pod =head1 METHODS
 #pod
-#pod Aside from the L</new> method, available method names are dynamically loaded
-#pod from the AvaTax WSDL file's operations and can be passed either a hash or
-#pod reference to a hash with the necessary parameters. In scalar context they
-#pod return a reference to a hash containing the results of the SOAP call; in list
-#pod context they return the results hashref and an
+#pod Aside from the L</new> method, L</services> attribute and
+#pod other attributes and methods consumed from
+#pod L<WebService::Avalara::AvaTax::Role::Connection|WebService::Avalara::AvaTax::Role::Connection>,
+#pod available method names are dynamically loaded from each
+#pod L</services>'
+#pod L<wsdl|WebService::Avalara::AvaTax::Role::Connection/wsdl>
+#pod attribute and can be passed either a hash or reference to a hash with the
+#pod necessary parameters. In scalar context they return a reference to a hash
+#pod containing the results of the SOAP call; in list context they return the
+#pod results hashref and an
 #pod L<XML::Compile::SOAP::Trace|XML::Compile::SOAP::Trace>
 #pod object suitable for debugging and exception handling.
 #pod
@@ -263,31 +136,23 @@ has _operation_name => ( is => 'rw', isa => Str, default => q{} );
 sub BUILD {
     my $self = shift;
 
-    # compile operations
-    my @operations;
-    for my $index ( 0 .. $#{ $self->_endpoints } ) {
-        my %soap_params
-            = map { ( $_ => $self->_endpoints->[$index]->$_ ) }
-            qw(port service);
-        $self->wsdl->compileCalls(
-            long_names => 1,
-            transport  => $self->_transports->[$index],
+    for my $service ( @{ $self->services } ) {
+        my %soap_params = map { ( $_ => $service->$_ ) } qw(port service);
+
+        $service->wsdl->compileCalls(
+            transport => $service->_transport,
             %soap_params,
         );
-        push @operations => $self->wsdl->operations(%soap_params);
-    }
+        for my $operation ( $service->wsdl->operations(%soap_params) ) {
 
-    # stash methods
-    for my $operation (@operations) {
-        my $method_name
-            = ( 2 > grep { $_->name eq $operation->name } @operations )
-            ? $operation->name
-            : $operation->longName;
-        $method_name =~ s/[#]//xms;
-        $method_name =~ s/ (?<= [[:alnum:]] ) ( [[:upper:]] ) /_\l$1/xmsg;
-        $method_name = lcfirst $method_name;
-        $self->_stash->add_symbol(
-            "&$method_name" => _method_closure($operation) );
+            # normalize operation name into a Perl method name
+            my $method_name = $operation->name;
+            $method_name =~ s/ (?<= [[:alnum:]] ) ( [[:upper:]] ) /_\l$1/xmsg;
+            $method_name = lcfirst $method_name;
+
+            $self->_stash->add_symbol(
+                "&$method_name" => _method_closure( $service, $operation ) );
+        }
     }
     return;
 }
@@ -317,15 +182,15 @@ const my %OPERATION_PARAMETER => (
 );
 
 sub _method_closure {
-    my $operation = shift;
+    my ( $service, $operation ) = @_;
     return sub {
         my ( $self, @parameters ) = @_;
-        my $wsdl = $self->wsdl;
 
-        $self->_operation_name( $operation->name );
+        my $wsdl = $service->wsdl;
+        $service->_current_operation_name( $operation->name );
+
         my ( $answer_ref, $trace ) = $wsdl->call(
-            $operation->serviceName . q{#}
-                . $operation->name => {
+            $operation->name => {
                 Profile => {
                     Client => "$PROGRAM_NAME," . ( $main::VERSION // q{} ),
                     Adapter => __PACKAGE__ . q{,} . ( $VERSION // q{} ),
@@ -337,7 +202,7 @@ sub _method_closure {
                     ? "@parameters"
                     : {@parameters},
                 },
-                },
+            },
         );
 
         #pod =pod
@@ -370,7 +235,7 @@ WebService::Avalara::AvaTax - Avalara SOAP interface as compiled Perl methods
 
 =head1 VERSION
 
-version 0.007
+version 0.008
 
 =head1 SYNOPSIS
 
@@ -386,24 +251,31 @@ version 0.007
 This class provides a Perl method API for
 L<Avalara AvaTax|http://developer.avalara.com/api-docs/soap>
 web services. The first call to any AvaTax SOAP operation uses
-L<XML::Compile::WSDL11|XML::Compile::WSDL11> to compile and execute against the
-specified Avalara AvaTax service; subsequent calls can vary the
-parameters but will use the same compiled code.
+L<XML::Compile::WSDL11|XML::Compile::WSDL11>
+to compile and execute against the specified Avalara AvaTax service;
+subsequent calls can vary the parameters but will use the same compiled code.
 
 =head1 METHODS
 
-Aside from the L</new> method, available method names are dynamically loaded
-from the AvaTax WSDL file's operations and can be passed either a hash or
-reference to a hash with the necessary parameters. In scalar context they
-return a reference to a hash containing the results of the SOAP call; in list
-context they return the results hashref and an
+Aside from the L</new> method, L</services> attribute and
+other attributes and methods consumed from
+L<WebService::Avalara::AvaTax::Role::Connection|WebService::Avalara::AvaTax::Role::Connection>,
+available method names are dynamically loaded from each
+L</services>'
+L<wsdl|WebService::Avalara::AvaTax::Role::Connection/wsdl>
+attribute and can be passed either a hash or reference to a hash with the
+necessary parameters. In scalar context they return a reference to a hash
+containing the results of the SOAP call; in list context they return the
+results hashref and an
 L<XML::Compile::SOAP::Trace|XML::Compile::SOAP::Trace>
 object suitable for debugging and exception handling.
 
 =head2 new
 
-Builds a new AvaTax web service client. See the L</ATTRIBUTES> section for
-description of its named parameters.
+Builds a new AvaTax web service client. Since this class consumes the
+L<WebService::Avalara::AvaTax::Role::Connection|WebService::Avalara::AvaTax::Role::Connection>
+role, please consult that module's documentation for a full list of attributes
+that can be set at construction.
 
 =head2 get_tax
 
@@ -584,11 +456,23 @@ Example:
         TextCase    => 'Upper',
     );
 
-=head2 tax_svc_is_authorized
+=head2 is_authorized
+
+Both
+L<WebService::Avalara::AvaTax::Service::Address|WebService::Avalara::AvaTax::Service::Address>
+and
+L<WebService::Avalara::AvaTax::Service::Tax|WebService::Avalara::AvaTax::Service::Tax>
+provide C<IsAuthorized> operations. However, since the latter is loaded last,
+only its version is called when you call this method. If you need to
+specifically call a particular service's C<IsAuthorized>, use the
+L<call|XML::Compile::WSDL11/Compilers>
+method on its
+L<wsdl|WebService::Avalara::AvaTax::Role::Connection/wsdl>
+attribute.
 
 Example:
 
-    my ( $answer_ref, $trace ) = $avatax->tax_svc_is_authorized(
+    my ( $answer_ref, $trace ) = $avatax->is_authorized(
         join ', ' => qw(
             Ping
             IsAuthorized
@@ -601,41 +485,24 @@ Example:
         ),
     );
 
-=head2 address_svc_is_authorized
+=head2 ping
 
-Example:
-
-    my ( $answer_ref, $trace ) = $avatax->address_svc_is_authorized(
-        join ', ' => qw(
-            Ping
-            IsAuthorized
-            Validate
-        ),
-    );
-
-=head2 tax_svc_ping
+Both
+L<WebService::Avalara::AvaTax::Service::Address|WebService::Avalara::AvaTax::Service::Address>
+and
+L<WebService::Avalara::AvaTax::Service::Tax|WebService::Avalara::AvaTax::Service::Tax>
+provide C<Ping> operations. However, since the latter is loaded last,
+only its version is called when you call this method. If you need to
+specifically call a particular service's C<Ping>, use the
+L<call|XML::Compile::WSDL11/Compilers>
+method on its
+L<wsdl|WebService::Avalara::AvaTax::Role::Connection/wsdl>
+attribute.
 
 Example:
 
     use List::Util 1.33 'any';
-    my ( $answer_ref, $trace ) = $avatax->tax_svc_ping;
-    for my $code ( $result_ref->{parameters}{PingResult}{ResultCode} ) {
-        if ( $code eq 'Success' ) { say $code; last }
-        if ( $code eq 'Warning' ) {
-            warn $result_ref->{parameters}{PingResult}{Messages};
-            last;
-        }
-        if ( any {$code eq $_} qw(Error Exception) ) {
-            die $result_ref->{parameters}{PingResult}{Messages};
-        }
-    }
-
-=head2 address_svc_ping
-
-Example:
-
-    use List::Util 1.33 'any';
-    my ( $answer_ref, $trace ) = $avatax->address_svc_ping;
+    my ( $answer_ref, $trace ) = $avatax->ping;
     for my $code ( $result_ref->{parameters}{PingResult}{ResultCode} ) {
         if ( $code eq 'Success' ) { say $code; last }
         if ( $code eq 'Warning' ) {
@@ -665,8 +532,8 @@ From L<Avalara API documentation|http://developer.avalara.com/api-docs/soap/appl
 
 The ApplyPayment method of the TaxSvc was originally designed to update an
 existing document record with a PaymentDate value. This function (and
-cash-basis accounting in general) is no longer supported, and will not work on
-new or existing accounts, but remains in the TaxSvc WSDL and some
+cash-basis accounting in general) is no longer supported, and will not work
+on new or existing accounts, but remains in the TaxSvc WSDL and some
 automatically built adaptors for backwards compatibility.
 
 =back
@@ -715,42 +582,29 @@ Example:
 
 =head1 ATTRIBUTES
 
-=head2 username
+=head2 services
 
-The Avalara email address used for authentication. Required.
+This module is really just a convenience wrapper around instances of
+L<WebService::Avalara::AvaTax::Service::Address|WebService::Avalara::AvaTax::Service::Address>
+and
+L<WebService::Avalara::AvaTax::Service::Tax|WebService::Avalara::AvaTax::Service::Tax>
+modules. As such this attribute is used to keep an array reference to
+instances of both classes, with the following attributes from L</new>
+passed to both:
 
-=head2 password
+=over
 
-The password used for Avalara authentication. Required.
+=item L<username|WebService::Avalara::AvaTax::Role::Connection/username>
 
-=head2 is_production
+=item L<password|WebService::Avalara::AvaTax::Role::Connection/password>
 
-A boolean value that indicates whether to connect to the production AvaTax
-services (true) or development (false). Defaults to false.
+=item L<is_production|WebService::Avalara::AvaTax::Role::Connection/is_production>
 
-=head2 debug
+=item L<user_agent|WebService::Avalara::AvaTax::Role::Connection/user_agent>
 
-When set to true, the L<Log::Report|Log::Report> dispatcher used by
-L<XML::Compile|XML::Compile> and friends is set to I<DEBUG> mode.
+=item L<debug|WebService::Avalara::AvaTax::Role::Connection/debug>
 
-=head2 user_agent
-
-An instance of an L<LWP::UserAgent|LWP::UserAgent> (sub-)class. You can
-use your own subclass to add features such as caching or enhanced logging.
-
-If you do not specify a C<user_agent> then we default to an
-L<LWPx::UserAgent::Cached|LWPx::UserAgent::Cached> with its C<ssl_opts>
-parameter set to C<< {verify_hostname => 0} >>.
-
-=head2 wsdl
-
-After construction, you can retrieve the created
-L<XML::Compile::WSDL11|XML::Compile::WSDL11> instance.
-
-Example:
-
-    my $wsdl = $avatax->wsdl;
-    my @soap_operations = map { $_->name } $wsdl->operations;
+=back
 
 =head1 SEE ALSO
 
