@@ -1,10 +1,10 @@
 package WebService::Avalara::AvaTax::Role::Connection;
 
-# ABSTRACT: Wrapper for Avalara AvaTax web services
+# ABSTRACT: Common attributes and methods for AvaTax
 
 use Modern::Perl '2011';    ## no critic (Modules::ProhibitUseQuotedVersion)
 
-our $VERSION = '0.004';     # TRIAL VERSION
+our $VERSION = '0.008';     # TRIAL VERSION
 use utf8;
 
 #pod =head1 SYNOPSIS
@@ -14,26 +14,25 @@ use utf8;
 #pod
 #pod =head1 DESCRIPTION
 #pod
-#pod This role integrates
-#pod L<Avalara AvaTax web services|http://developer.avalara.com/api-docs/soap>.
+#pod This role factors out the common attributes and methods used by the
+#pod L<Avalara AvaTax|http://developer.avalara.com/api-docs/soap>
+#pod web service interface.
 #pod
 #pod =cut
 
-use Const::Fast;
-use English '-no_match_vars';
 use Log::Report;
+use LWP::UserAgent;
 use LWPx::UserAgent::Cached;
-use Scalar::Util 'blessed';
-use Sys::Hostname;
+use Moo::Role;
+use MooX::Types::MooseLike::Email 'EmailAddressLoose';
 use Types::Standard qw(Bool InstanceOf Str);
+use Types::URI 'Uri';
 use URI;
 use XML::Compile::SOAP::WSS;
-use XML::Compile::SOAP11;
-use XML::Compile::SOAP12;
-use XML::Compile::WSDL11;
 use XML::Compile::Transport::SOAPHTTP;
-use MooX::Types::MooseLike::Email 'EmailAddressLoose';
-use Moo::Role;
+use XML::Compile::WSDL11;
+use XML::Compile::WSS;
+use namespace::clean;
 
 #pod =attr username
 #pod
@@ -51,55 +50,44 @@ has username => ( is => 'ro', isa => EmailAddressLoose, required => 1 );
 
 has password => ( is => 'ro', isa => Str, required => 1 );
 
-#pod =attr soap_service
+#pod =attr is_production
 #pod
-#pod The SOAP service name for AvaTax. Defaults to C<TaxSvc>.
+#pod A boolean value that indicates whether to connect to the production AvaTax
+#pod services (true) or development (false). Defaults to false.
 #pod
 #pod =cut
 
-has soap_service => ( is => 'ro', isa => Str, default => 'TaxSvc' );
+has is_production => ( is => 'ro', isa => Bool, default => 0 );
 
-#pod =attr soap_port
+#pod =attr uri
 #pod
-#pod The SOAP service name for AvaTax. Defaults to C<TaxSvcSoap>.
+#pod The L<URI|URI> of the WSDL file used to define the web service.
+#pod Consumer classes are expected to set this attribute.
+#pod As a convenience this can also be set with anything that
+#pod L<Types::URI|Types::URI> can coerce into a C<Uri>.
 #pod
 #pod =cut
 
-has soap_port => ( is => 'ro', isa => Str, default => 'TaxSvcSoap' );
+has uri => ( is => 'lazy', isa => Uri, coerce => 1 );
 
-has _wss => (
-    is      => 'ro',
-    isa     => InstanceOf ['XML::Compile::SOAP::WSS'],
-    default => sub { XML::Compile::SOAP::WSS->new },
-);
-
-has _auth => ( is => 'lazy', isa => InstanceOf ['XML::Compile::WSS'] );
-
-sub _build__auth {
-    my $self = shift;
-    my $wss  = $self->_wss;
-    return $wss->basicAuth( map { ( $_ => $self->$_ ) }
-            qw(username password) );
-}
-
-#pod =attr endpoint
+#pod =attr port
 #pod
-#pod A L<URI|URI> object indicating the AvaTax WSDL file to load. Defaults to
-#pod L<https://development.avalara.net/tax/taxsvc.wsdl>. For production API access
-#pod one should set this to L<https://avatax.avalara.net/tax/taxsvc.wsdl>.
+#pod The SOAP port identifier (not to be confused with the TCP/IP port) used in the
+#pod WSDL file at L</uri>.
+#pod Consumer classes are expected to set this attribute.
 #pod
 #pod =cut
 
-has endpoint => (
-    is     => 'lazy',
-    isa    => InstanceOf ['URI'],
-    coerce => sub {
-        defined blessed( $_[0] )
-            && $_[0]->isa('URI') ? $_[0] : URI->new( $_[0] );
-    },
-    default =>
-        sub { URI->new('https://development.avalara.net/tax/taxsvc.wsdl') },
-);
+has port => ( is => 'ro', isa => Str );
+
+#pod =attr service
+#pod
+#pod The SOAP service name used in the WSDL file at L</uri>.
+#pod Consumer classes are expected to set this attribute.
+#pod
+#pod =cut
+
+has service => ( is => 'ro', isa => Str );
 
 #pod =attr debug
 #pod
@@ -116,69 +104,6 @@ has debug => (
         sub { dispatcher( mode => ( $_[1] ? 'DEBUG' : 'NORMAL' ), 'ALL' ) },
 );
 
-#pod =attr transport
-#pod
-#pod An L<XML::Compile::Transport::SOAPHTTP|XML::Compile::Transport::SOAPHTTP>
-#pod object used to make SOAP calls. By default it uses C<user_agent> along with
-#pod C<wsdl>'s C<endPoint> for the address, and adds a handler for C<POST>s to
-#pod the endpoint to add the appropriate C<SOAPAction> HTTP header.
-#pod
-#pod =cut
-
-has transport =>
-    ( is => 'lazy', isa => InstanceOf ['XML::Compile::Transport::SOAPHTTP'] );
-
-sub _build_transport {
-    my $self = shift;
-    my $wss  = $self->_wss;
-    my $wsdl = $self->wsdl;
-    my $auth = $self->_auth;
-
-    my $user_agent  = $self->user_agent;
-    my %soap_params = (
-        service => $self->soap_service,
-        port    => $self->soap_port,
-    );
-    my $endpoint_uri = URI->new( $wsdl->endPoint(%soap_params) );
-    $user_agent->add_handler(
-        request_prepare => sub {
-            $_[0]->header( SOAPAction =>
-                    $wsdl->operation( $self->_operation_name, %soap_params )
-                    ->soapAction, );
-        },
-        (   m_method => 'POST',
-            map { ( "m_$_" => $endpoint_uri->$_ ) } qw(scheme host_port path),
-        ),
-    );
-    return XML::Compile::Transport::SOAPHTTP->new(
-        address    => $endpoint_uri,
-        user_agent => $self->user_agent,
-    );
-}
-
-has _operation_name => ( is => 'rw', isa => Str, default => q{} );
-
-#pod =method call
-#pod
-#pod Given an operation name and parameters, makes a SOAP call.
-#pod
-#pod =cut
-
-sub call {
-    my ( $self, $operation, @params ) = @_;
-    $self->_operation_name($operation);
-    return $self->wsdl->call(
-        $operation => {
-            Profile => {
-                Client => "$PROGRAM_NAME," . ( $main::VERSION // q{} ),
-                Adapter => __PACKAGE__ . q{,} . ( $VERSION // q{} ),
-                Machine => hostname(),
-            },
-            parameters => {@params},
-        },
-    );
-}
-
 #pod =attr user_agent
 #pod
 #pod An instance of an L<LWP::UserAgent|LWP::UserAgent> (sub-)class. You can
@@ -198,12 +123,84 @@ has user_agent => (
     },
 );
 
-has wsdl => ( is => 'lazy', isa => InstanceOf ['XML::Compile::WSDL11'] );
+#pod =attr wsdl
+#pod
+#pod After construction, you can retrieve the created
+#pod L<XML::Compile::WSDL11|XML::Compile::WSDL11> instance.
+#pod
+#pod Example:
+#pod
+#pod     my $wsdl = $avatax->wsdl;
+#pod     my @soap_operations = map { $_->name } $wsdl->operations;
+#pod
+#pod =cut
 
-sub _build_wsdl {
+has wsdl => (
+    is       => 'lazy',
+    isa      => InstanceOf ['XML::Compile::WSDL11'],
+    init_arg => undef,
+    default  => sub {
+        XML::Compile::WSDL11->new(
+            $_[0]->user_agent->get( $_[0]->uri )->content );
+    },
+);
+
+has _wss => (
+    is       => 'ro',
+    isa      => InstanceOf ['XML::Compile::SOAP::WSS'],
+    default  => sub { XML::Compile::SOAP::WSS->new },
+    init_arg => undef,
+);
+
+has _auth => (
+    is       => 'lazy',
+    isa      => InstanceOf ['XML::Compile::WSS'],
+    init_arg => undef,
+);
+
+sub _build__auth {
     my $self = shift;
-    return XML::Compile::WSDL11->new(
-        $self->user_agent->get( $self->endpoint )->content );
+    my $wss  = $self->_wss;
+    return $wss->basicAuth( map { ( $_ => $self->$_ ) }
+            qw(username password) );
+}
+
+has _current_operation_name => ( is => 'rw', isa => Str, default => q{} );
+
+has _transport => (
+    is       => 'lazy',
+    isa      => InstanceOf ['XML::Compile::Transport::SOAPHTTP'],
+    init_arg => undef,
+);
+
+sub _build__transport {
+    my $self = shift;
+
+    my $wss  = $self->_wss;
+    my $wsdl = $self->wsdl;
+    my $auth = $self->_auth;
+
+    my %soap_params  = map { ( $_ => $self->$_ ) } qw(port service);
+    my $endpoint_uri = URI->new( $wsdl->endPoint(%soap_params) );
+    my $user_agent   = $self->user_agent;
+
+    $user_agent->add_handler(
+        request_prepare => sub {
+            $_[0]->header(
+                SOAPAction =>
+                    $wsdl->operation( $self->_current_operation_name,
+                    %soap_params )->soapAction,
+            );
+        },
+        (   m_method => 'POST',
+            map { ( "m_$_" => $endpoint_uri->$_ ) } qw(scheme host_port path),
+        ),
+    );
+
+    return XML::Compile::Transport::SOAPHTTP->new(
+        address    => $endpoint_uri,
+        user_agent => $user_agent,
+    );
 }
 
 1;
@@ -219,11 +216,11 @@ cpants kwalitee diff irc mailto metadata placeholders metacpan
 
 =head1 NAME
 
-WebService::Avalara::AvaTax::Role::Connection - Wrapper for Avalara AvaTax web services
+WebService::Avalara::AvaTax::Role::Connection - Common attributes and methods for AvaTax
 
 =head1 VERSION
 
-version 0.004
+version 0.008
 
 =head1 SYNOPSIS
 
@@ -232,8 +229,9 @@ version 0.004
 
 =head1 DESCRIPTION
 
-This role integrates
-L<Avalara AvaTax web services|http://developer.avalara.com/api-docs/soap>.
+This role factors out the common attributes and methods used by the
+L<Avalara AvaTax|http://developer.avalara.com/api-docs/soap>
+web service interface.
 
 =head1 ATTRIBUTES
 
@@ -245,32 +243,34 @@ The Avalara email address used for authentication. Required.
 
 The password used for Avalara authentication. Required.
 
-=head2 soap_service
+=head2 is_production
 
-The SOAP service name for AvaTax. Defaults to C<TaxSvc>.
+A boolean value that indicates whether to connect to the production AvaTax
+services (true) or development (false). Defaults to false.
 
-=head2 soap_port
+=head2 uri
 
-The SOAP service name for AvaTax. Defaults to C<TaxSvcSoap>.
+The L<URI|URI> of the WSDL file used to define the web service.
+Consumer classes are expected to set this attribute.
+As a convenience this can also be set with anything that
+L<Types::URI|Types::URI> can coerce into a C<Uri>.
 
-=head2 endpoint
+=head2 port
 
-A L<URI|URI> object indicating the AvaTax WSDL file to load. Defaults to
-L<https://development.avalara.net/tax/taxsvc.wsdl>. For production API access
-one should set this to L<https://avatax.avalara.net/tax/taxsvc.wsdl>.
+The SOAP port identifier (not to be confused with the TCP/IP port) used in the
+WSDL file at L</uri>.
+Consumer classes are expected to set this attribute.
+
+=head2 service
+
+The SOAP service name used in the WSDL file at L</uri>.
+Consumer classes are expected to set this attribute.
 
 =head2 debug
 
 When set to true, the L<Log::Report|Log::Report> dispatcher used by
 L<XML::Compile|XML::Compile> and friends is set to I<DEBUG> mode.
 
-=head2 transport
-
-An L<XML::Compile::Transport::SOAPHTTP|XML::Compile::Transport::SOAPHTTP>
-object used to make SOAP calls. By default it uses C<user_agent> along with
-C<wsdl>'s C<endPoint> for the address, and adds a handler for C<POST>s to
-the endpoint to add the appropriate C<SOAPAction> HTTP header.
-
 =head2 user_agent
 
 An instance of an L<LWP::UserAgent|LWP::UserAgent> (sub-)class. You can
@@ -280,11 +280,15 @@ If you do not specify a C<user_agent> then we default to an
 L<LWPx::UserAgent::Cached|LWPx::UserAgent::Cached> with its C<ssl_opts>
 parameter set to C<< {verify_hostname => 0} >>.
 
-=head1 METHODS
+=head2 wsdl
 
-=head2 call
+After construction, you can retrieve the created
+L<XML::Compile::WSDL11|XML::Compile::WSDL11> instance.
 
-Given an operation name and parameters, makes a SOAP call.
+Example:
+
+    my $wsdl = $avatax->wsdl;
+    my @soap_operations = map { $_->name } $wsdl->operations;
 
 =head1 SUPPORT
 
